function transportAction(json)
{
    let settings = json.payload.settings;
    let image = new sdImage(json.context);
    let longHoldTimeout = false;

    function updateImage()
    {
        let foreColor = settings.foreColor;
        let symbol;
        switch (settings.action)
        {
            case "play":
                symbol = "play";
                foreColor = C.transport.state == "playing" ? "#00FF00" : settings.foreColor;
                break;

            case "pause":
                symbol = "pause";
                foreColor = C.transport.state == "paused" ? "#FF5500" : settings.foreColor;
                break;

            case "stop":
                symbol = "stop";
                break;

            case "playPause":
            case "playStop":
                switch (C.transport.state)
                {
                    case "playing":
                        symbol = "play";
                        foreColor = "#00FF00";
                        break;

                    case "paused":
                        symbol = "pause";
                        foreColor = "#FF5500";
                        break;

                    case "stopped":
                        symbol = "play";
                        break;
                }
                break;
        }

        image.disabled = C.state != "connected";
        image.setSvg(`images/transport_${symbol}.svg`, {
            "#background.fill": settings.backColor,
            "#848484": foreColor,
        });
    }

    // Callback from Cantabile that transport state has changed
    function onStateChanged()
    {
        updateImage();
    }

    function startWatching()
    {
        stopWatching();

        C.transport.on('stateChanged', onStateChanged);
    }

    function stopWatching()
    {
        C.transport.off('stateChanged', onStateChanged);
    }

    function wantsLongPress()
    {
        switch (settings.action)
        {
            case "playPause":
            case "playStop":
                return true;
        }
        return false;
    }

    function onPress()
    {
        switch (settings.action)
        {
            case "play": C.transport.play(); break;
            case "pause": C.transport.pause(); break;
            case "stop": C.transport.stop(); break;
            case "playPause": C.transport.togglePlayPause(); break;
            case "playStop": C.transport.togglePlayStop(); break;
        }
    }

    function onLongPress()
    {
        switch (settings.action)
        {
            case "playPause": C.transport.stop(); break;
            case "playStop": C.transport.pause(); break;
        }
    }

    return {

        onKeyUp: function (jsonEvent)
        {
            if (!wantsLongPress() || longHoldTimeout)
            {
                clearTimeout(longHoldTimeout);
                longHoldTimeout = null;
                onPress();
            }
        },
        onKeyDown: function (jsonEvent)
        {
            if (wantsLongPress())
            {
                longHoldTimeout = setTimeout(function() {
                    // Long press, toggle auto-record on/off
                    longHoldTimeout = null;
                    onLongPress();
                }, 1000);
            }
        },
        onDidReceiveSettings: function (jsonEvent)
        {
            settings = $SD.api.initDefaultSettings(jsonEvent, {
                action: "play",
                backColor: "#000000",
                foreColor: "#848484",
            });

            updateImage();
            startWatching();
        },
        onWillDisappear: function (jsonEvent)
        {
            C.transport.removeListener('stateChanged', onStateChanged)
            stopWatching();
        },
        onConnectionStateChanged: function ()
        {
            updateImage();
        }
    }
}